@@ -23,7 +23,6 @@ import (
 	templatev1 "github.com/openshift/api/template/v1"
 	imagev1client "github.com/openshift/client-go/image/clientset/versioned/typed/image/v1"
 
-	"github.com/openshift/cluster-samples-operator/pkg/cache"
 	operator "github.com/openshift/cluster-samples-operator/pkg/operatorstatus"
 	"github.com/openshift/cluster-samples-operator/pkg/util"
 )
@@ -50,30 +49,11 @@ func TestWrongSampleResourceName(t *testing.T) {
 	validate(true, err, "", cfg, []v1.ConfigConditionType{}, []corev1.ConditionStatus{}, t)
 }
 
-func TestNoArchOrDist(t *testing.T) {
-	h, cfg, event := setup()
-	err := h.Handle(event)
-	// image in progress (4th entry, array index 3) should still be false when there is no content ... a la z or ppc
-	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
-	validate(true, err, "", cfg, conditions, statuses, t)
-	err = h.Handle(event)
-	// and on a subsequent event, exists should still be false since we did not create any content previously
-	validate(true, err, "", cfg, conditions, statuses, t)
-}
-
 func TestWithDist(t *testing.T) {
 	h, cfg, event := setup()
-	err := h.Handle(event)
-	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
-	validate(true, err, "", cfg, conditions, statuses, t)
-	err = h.Handle(event)
-	// image in progress (4th entry, array index 3) should still be false when there is no content ... a la z or ppc
-	statuses = []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
-	validate(true, err, "", cfg, conditions, statuses, t)
 	mimic(&h, x86ContentRootDir)
-	err = h.Handle(event)
-	// with content present, image im progress should now be true
-	statuses = []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+	err := h.Handle(event)
+	statuses := []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
 }
 
@@ -92,12 +72,7 @@ func TestWithArchDist(t *testing.T) {
 
 	mimic(&h, x86ContentRootDir)
 	err := h.Handle(event)
-	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
-	validate(true, err, "", cfg,
-		conditions,
-		statuses, t)
-	err = h.Handle(event)
-	statuses = []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+	statuses := []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg,
 		conditions,
 		statuses, t)
@@ -111,7 +86,7 @@ func TestWithArch(t *testing.T) {
 		v1.PPCArchitecture,
 	}
 	err := h.Handle(event)
-	validateArchOverride(true, err, "", cfg, conditions, []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t, v1.PPCArchitecture)
+	validateArchOverride(true, err, "", cfg, conditions, []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t, v1.PPCArchitecture)
 }
 
 func TestWithBadArch(t *testing.T) {
@@ -150,18 +125,7 @@ func TestManagementState(t *testing.T) {
 	cfg.ResourceVersion = "2"
 	cfg.Spec.ManagementState = operatorsv1api.Managed
 	err = h.Handle(event)
-	statuses = []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
-	validate(true, err, "", cfg, conditions, statuses, t)
-
-	err = h.Handle(event)
-	// event after in progress set to true, sets exists to true
-	statuses = []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
-	validate(true, err, "", cfg, conditions, statuses, t)
-
-	// event after exists is true that should trigger samples upsert
-	err = h.Handle(event)
-	// event after in progress set to true
-	statuses = []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+	statuses = []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
 
 	for _, key := range iskeys {
@@ -219,7 +183,7 @@ func TestSkipped(t *testing.T) {
 	mimic(&h, x86ContentRootDir)
 
 	err := h.Handle(event)
-	validate(true, err, "", cfg, conditions, []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
+	validate(true, err, "", cfg, conditions, []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
 
 	fakeisclient := h.imageclientwrapper.(*fakeImageStreamClientWrapper)
 	for _, key := range iskeys {
@@ -242,10 +206,21 @@ func TestSkipped(t *testing.T) {
 
 	// also, even with an import error, on an imagestream event, the import error should be cleared out
 	importerror := util.Condition(cfg, v1.ImportImageErrorsExist)
-	importerror.Reason = "foo "
-	importerror.Message = "<imagestream/foo> import failed <imagestream/foo>"
 	importerror.Status = corev1.ConditionTrue
 	util.ConditionUpdate(cfg, importerror)
+
+	fakecmclient := h.configmapclientwrapper.(*fakeConfigMapClientWrapper)
+	cm := &corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "foo",
+		},
+		Data: map[string]string{
+			"foo": "could not import",
+		},
+		BinaryData: nil,
+	}
+	fakecmclient.configMaps["foo"] = cm
+
 	event.Object = &imagev1.ImageStream{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      "foo",
@@ -264,8 +239,19 @@ func TestSkipped(t *testing.T) {
 		},
 	}
 	h.Handle(event)
+	_, stillHasCM := fakecmclient.configMaps["foo"]
+	if stillHasCM {
+		t.Fatalf("clean imagestream did not result in configmap getting deleted")
+	}
+	event.Deleted = true
+	event.Object = &corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "foo",
+		},
+	}
+	h.Handle(event)
 	importerror = util.Condition(cfg, v1.ImportImageErrorsExist)
-	if len(importerror.Reason) > 0 || importerror.Status == corev1.ConditionTrue {
+	if importerror.Status == corev1.ConditionTrue {
 		t.Fatalf("skipped imagestream still reporting error %#v", importerror)
 	}
 }
@@ -303,16 +289,7 @@ func TestProcessed(t *testing.T) {
 	err := h.Handle(event)
 	validate(true, err, "", cfg,
 		conditions,
-		[]corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
-
-	err = h.Handle(event)
-	validate(true, err, "", cfg,
-		conditions,
-		[]corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
-	err = h.Handle(event)
-	validate(true, err, "", cfg,
-		conditions,
-		[]corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
+		[]corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
 
 	fakeisclient := h.imageclientwrapper.(*fakeImageStreamClientWrapper)
 	for _, key := range iskeys {
@@ -349,15 +326,14 @@ func TestProcessed(t *testing.T) {
 	// make sure registries are updated after already updating from the defaults
 	cfg.Spec.SamplesRegistry = "bar.io"
 	cfg.ResourceVersion = "2"
-	// fake out that the samples completed updating
-	progressing := util.Condition(cfg, v1.ImageChangesInProgress)
-	progressing.Status = corev1.ConditionFalse
-	util.ConditionUpdate(cfg, progressing)
+	// clear out config map tracking
+	fakecfgmapclient := h.configmapclientwrapper.(*fakeConfigMapClientWrapper)
+	fakecfgmapclient.configMaps = map[string]*corev1.ConfigMap{}
 
 	err = h.Handle(event)
 	validate(true, err, "", cfg,
 		conditions,
-		[]corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
+		[]corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
 	is, _ = fakeisclient.Get("foo")
 	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
 		t.Fatalf("stream repo not updated %#v, %#v", is, h)
@@ -374,17 +350,15 @@ func TestProcessed(t *testing.T) {
 	// make sure registries are updated when sampleRegistry is the form of host/path
 	cfg.Spec.SamplesRegistry = "foo.io/bar"
 	cfg.ResourceVersion = "3"
-	// fake out that the samples completed updating
-	progressing = util.Condition(cfg, v1.ImageChangesInProgress)
-	progressing.Status = corev1.ConditionFalse
-	util.ConditionUpdate(cfg, progressing)
+	// clear out config map tracking
+	fakecfgmapclient.configMaps = map[string]*corev1.ConfigMap{}
 	// reset operator image to clear out previous registry image override
 	mimic(&h, x86ContentRootDir)
 
 	err = h.Handle(event)
 	validate(true, err, "", cfg,
 		conditions,
-		[]corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
+		[]corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
 	is, _ = fakeisclient.Get("foo")
 	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
 		t.Fatalf("stream repo not updated %#v, %#v", is, h)
@@ -401,17 +375,15 @@ func TestProcessed(t *testing.T) {
 	// make sure registries are updated when sampleRegistry is the form of host:port/path
 	cfg.Spec.SamplesRegistry = "foo.io:1111/bar"
 	cfg.ResourceVersion = "4"
-	// fake out that the samples completed updating
-	progressing = util.Condition(cfg, v1.ImageChangesInProgress)
-	progressing.Status = corev1.ConditionFalse
-	util.ConditionUpdate(cfg, progressing)
+	// clear out config map tracking
+	fakecfgmapclient.configMaps = map[string]*corev1.ConfigMap{}
 	// reset operator image to clear out previous registry image override
 	mimic(&h, x86ContentRootDir)
 
 	err = h.Handle(event)
 	validate(true, err, "", cfg,
 		conditions,
-		[]corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
+		[]corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
 	is, _ = fakeisclient.Get("foo")
 	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
 		t.Fatalf("stream repo not updated %#v, %#v", is, h)
@@ -427,11 +399,8 @@ func TestProcessed(t *testing.T) {
 	// make sure registries are updated when switch back to default
 	cfg.ResourceVersion = "5"
 	cfg.Spec.SamplesRegistry = ""
-	// also make sure processing occurs for disruptive config change even if progressing==true
-	progressing = util.Condition(cfg, v1.ImageChangesInProgress)
-	// fake out that the samples completed updating
-	progressing.Status = corev1.ConditionFalse
-	util.ConditionUpdate(cfg, progressing)
+	// clear out config map tracking
+	fakecfgmapclient.configMaps = map[string]*corev1.ConfigMap{}
 	// reset operator image to clear out previous registry image override
 	mimic(&h, x86ContentRootDir)
 
@@ -454,10 +423,10 @@ func TestImageStreamEvent(t *testing.T) {
 	h, cfg, event := setup()
 	mimic(&h, x86ContentRootDir)
 	err := h.Handle(event)
-	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+	statuses := []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
 	// expedite the stream events coming in
-	cache.ClearUpsertsCache()
+	//cache.ClearUpsertsCache()
 
 	tagVersion := int64(1)
 	is := &imagev1.ImageStream{
@@ -505,17 +474,11 @@ func TestImageStreamEvent(t *testing.T) {
 		t.Fatalf("is did not reach client %s: %#v", "foo", h)
 	}
 
-	// mimic img change condition event that sets exists to true
-	err = h.Handle(event)
-	statuses[0] = corev1.ConditionTrue
-	validate(true, err, "", cfg, conditions, statuses, t)
-
 	// with the update above, now process both of the imagestreams and see the in progress condition
 	// go false
 	is.Annotations[v1.SamplesVersionAnnotation] = h.version
 	h.processImageStreamWatchEvent(is, false)
 	validate(true, err, "", cfg, conditions, statuses, t)
-	statuses[3] = corev1.ConditionFalse
 	is = &imagev1.ImageStream{
 		ObjectMeta: metav1.ObjectMeta{
 			Name: "bar",
@@ -574,6 +537,7 @@ func TestImageStreamEvent(t *testing.T) {
 		},
 	}
 	h.processImageStreamWatchEvent(is, false)
+	h.processImageCondition()
 	validate(true, err, "", cfg, conditions, statuses, t)
 }
 
@@ -581,10 +545,8 @@ func TestImageStreamErrorRetry(t *testing.T) {
 	h, cfg, event := setup()
 	mimic(&h, x86ContentRootDir)
 	err := h.Handle(event)
-	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+	statuses := []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
-	// expedite the stream events coming in
-	cache.ClearUpsertsCache()
 
 	tagVersion := int64(1)
 	is := &imagev1.ImageStream{
@@ -640,10 +602,22 @@ func TestImageStreamErrorRetry(t *testing.T) {
 
 	h.processImageStreamWatchEvent(is, false)
 
+	fakecmclient := h.configmapclientwrapper.(*fakeConfigMapClientWrapper)
+	cm, exists := fakecmclient.configMaps[is.Name]
+	if !exists {
+		t.Fatalf("no associated configmap for %s", is.Name)
+	}
+	event = util.Event{Object: cm}
+	h.Handle(event)
+
 	if !util.ConditionTrue(cfg, v1.ImportImageErrorsExist) {
 		t.Fatalf("Import Error Condition not true: %#v", cfg)
 	}
 
+	if !h.imageStreamHasErrors(is.Name) {
+		t.Fatalf("Import errors not registered in config map for %s", is.Name)
+	}
+
 	fakeisclient := h.imageclientwrapper.(*fakeImageStreamClientWrapper)
 	fakeimporter := fakeisclient.ImageStreamImports("foo").(*fakeImageStreamImporter)
 
@@ -653,6 +627,9 @@ func TestImageStreamErrorRetry(t *testing.T) {
 
 	initialImportErrorLastUpdateTime := util.Condition(cfg, v1.ImportImageErrorsExist).LastUpdateTime
 	h.processImageStreamWatchEvent(is, false)
+	cm, _ = fakecmclient.configMaps[is.Name]
+	event = util.Event{Object: cm}
+	h.Handle(event)
 	// refetch to see if updated
 	importError := util.Condition(cfg, v1.ImportImageErrorsExist)
 	if !importError.LastUpdateTime.Equal(&initialImportErrorLastUpdateTime) {
@@ -687,8 +664,20 @@ func TestImageStreamErrorRetry(t *testing.T) {
 
 	h.processImageStreamWatchEvent(is, false)
 
+	_, stillHasCM := fakecmclient.configMaps["foo"]
+	if stillHasCM {
+		t.Fatalf("clean imagestream did not result in configmap getting deleted")
+	}
+	event.Deleted = true
+	event.Object = &corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "foo",
+		},
+	}
+	h.Handle(event)
+
 	if util.ConditionTrue(cfg, v1.ImportImageErrorsExist) {
-		t.Fatalf("Import Error Condition not true: %#v", cfg)
+		t.Fatalf("Import Error Condition still true: %#v", cfg)
 	}
 }
 
@@ -696,13 +685,13 @@ func TestTemplateEvent(t *testing.T) {
 	h, cfg, event := setup()
 	mimic(&h, x86ContentRootDir)
 	err := h.Handle(event)
-	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+	statuses := []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
 	err = h.Handle(event)
 	statuses[0] = corev1.ConditionTrue
 	validate(true, err, "", cfg, conditions, statuses, t)
 	// expedite the template events coming in
-	cache.ClearUpsertsCache()
+	//cache.ClearUpsertsCache()
 
 	template := &templatev1.Template{
 		ObjectMeta: metav1.ObjectMeta{
@@ -724,7 +713,7 @@ func setup() (Handler, *v1.Config, util.Event) {
 	cfg, _ := h.CreateDefaultResourceIfNeeded(nil)
 	cfg = h.initConditions(cfg)
 	h.crdwrapper.(*fakeCRDWrapper).cfg = cfg
-	cache.ClearUpsertsCache()
+	//cache.ClearUpsertsCache()
 	return h, cfg, util.Event{Object: cfg}
 }
 
@@ -732,14 +721,12 @@ func TestTemplateRemovedFromPayload(t *testing.T) {
 	h, _, _ := setup()
 	mimic(&h, x86ContentRootDir)
 
-	_, filePath, doUpsert, updateCfgOnly, err := h.prepSamplesWatchEvent("template", "no-longer-exists", map[string]string{}, false)
+	_, filePath, doUpsert, err := h.prepSamplesWatchEvent("template", "no-longer-exists", map[string]string{}, false)
 	switch {
 	case len(filePath) > 0:
 		t.Fatalf("should not have returned a file path")
 	case doUpsert:
 		t.Fatalf("do upsert should not be true")
-	case updateCfgOnly:
-		t.Fatalf("update cfg only should not be true")
 	case err != nil:
 		t.Fatalf("got unexpected err %s", err.Error())
 	}
@@ -748,12 +735,20 @@ func TestTemplateRemovedFromPayload(t *testing.T) {
 func TestImageStreamRemovedFromPayloadWithProgressingErrors(t *testing.T) {
 	h, cfg, _ := setup()
 	mimic(&h, x86ContentRootDir)
-	progressing := util.Condition(cfg, v1.ImageChangesInProgress)
-	progressing.Reason = "foo "
 	errors := util.Condition(cfg, v1.ImportImageErrorsExist)
-	errors.Reason = "bar "
-	util.ConditionUpdate(cfg, progressing)
 	util.ConditionUpdate(cfg, errors)
+	cm := &corev1.ConfigMap{}
+	cm.Name = "foo"
+	fakeconfigmapclient := h.configmapclientwrapper.(*fakeConfigMapClientWrapper)
+	fakeconfigmapclient.Create(cm)
+	cm = &corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "bar",
+		},
+		Data: map[string]string{"bar": "could not import"},
+	}
+	fakeconfigmapclient.Create(cm)
+
 	fakefile := h.Filefinder.(*fakeResourceFileLister)
 	fakefile.files = map[string][]fakeFileInfo{}
 	tagVersion := int64(1)
@@ -789,15 +784,18 @@ func TestImageStreamRemovedFromPayloadWithProgressingErrors(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	progressing = util.Condition(cfg, v1.ImageChangesInProgress)
-	if strings.Contains(progressing.Reason, "foo") {
-		t.Fatal("progressing still has foo after it was no longer in payload")
+	cm, err = fakeconfigmapclient.Get("foo")
+	if cm != nil {
+		t.Fatal("still tracking foo after it was no longer in payload")
 	}
 	is.Name = "bar"
 	err = h.processImageStreamWatchEvent(is, false)
-	errors = util.Condition(cfg, v1.ImportImageErrorsExist)
-	if strings.Contains(errors.Reason, "bar") {
-		t.Fatal("import errors still has bar after it was no longer in payload")
+	cm, err = fakeconfigmapclient.Get("bar")
+	if cm != nil {
+		t.Fatal("still tracking bar after it was no longer in payload")
+	}
+	if util.ConditionTrue(cfg, v1.ImportImageErrorsExist) {
+		t.Fatal("still tracking import error true after no longer in payload")
 	}
 
 }
@@ -829,7 +827,7 @@ func TestImageGetError(t *testing.T) {
 		fakeisclient := h.imageclientwrapper.(*fakeImageStreamClientWrapper)
 		fakeisclient.geterrors = map[string]error{"foo": iserr}
 
-		statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+		statuses := []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 		err := h.Handle(event)
 		if !kerrors.IsNotFound(iserr) {
 			statuses[0] = corev1.ConditionUnknown
@@ -1014,10 +1012,15 @@ func TestImageStreamImportError(t *testing.T) {
 		dir := h.GetBaseDir(v1.X86Architecture, cfg)
 		files, _ := h.Filefinder.List(dir)
 		h.processFiles(dir, files, cfg)
-		progressing := util.Condition(cfg, v1.ImageChangesInProgress)
-		progressing.Status = corev1.ConditionTrue
-		progressing.Reason = is.Name + " "
-		util.ConditionUpdate(cfg, progressing)
+
+		cm := &corev1.ConfigMap{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: is.Name,
+			},
+		}
+		fakecmclient := h.configmapclientwrapper.(*fakeConfigMapClientWrapper)
+		fakecmclient.configMaps[is.Name] = cm
+
 		needCreds := util.Condition(cfg, v1.ImportCredentialsExist)
 		needCreds.Status = corev1.ConditionTrue
 		util.ConditionUpdate(cfg, needCreds)
@@ -1025,14 +1028,108 @@ func TestImageStreamImportError(t *testing.T) {
 		if err != nil {
 			t.Fatalf("processImageStreamWatchEvent error %#v for stream %#v", err, is)
 		}
+		event := util.Event{Object: cm}
+		h.Handle(event)
 		if util.ConditionFalse(cfg, v1.ImportImageErrorsExist) {
 			t.Fatalf("processImageStreamWatchEvent did not set import error to true %#v for stream %#v", cfg, is)
 		}
 
-		importErr := util.Condition(cfg, v1.ImportImageErrorsExist)
-		if len(importErr.Reason) == 0 || !util.NameInReason(cfg, importErr.Reason, is.Name) {
-			t.Fatalf("processImageStreamWatchEvent did not set import error reason field %#v for stream %#v", cfg, is)
+		cm, err = fakecmclient.Get(is.Name)
+		if err != nil {
+			t.Fatalf("error on cm get %s: %s", is.Name, err.Error())
 		}
+		if cm == nil {
+			t.Fatalf("cm get nil %s", is.Name)
+		}
+		if cm.Data == nil || len(cm.Data) == 0 {
+			t.Fatalf("processImageStreamWatchEvent did not put error for imagestream %s in configmap: %#v", is.Name, cm)
+		}
+	}
+}
+
+func TestImageStreamTagImportErrorRecovery(t *testing.T) {
+	two := int64(2)
+	one := int64(1)
+	stream := &imagev1.ImageStream{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "foo",
+			Annotations: map[string]string{
+				v1.SamplesVersionAnnotation: TestVersion,
+			},
+		},
+		Spec: imagev1.ImageStreamSpec{
+			Tags: []imagev1.TagReference{
+				{
+					Name:       "1",
+					Generation: &two,
+				},
+				{
+					Name:       "2",
+					Generation: &one,
+				},
+			},
+		},
+		Status: imagev1.ImageStreamStatus{
+			Tags: []imagev1.NamedTagEventList{
+				{
+					Tag: "1",
+					Items: []imagev1.TagEvent{
+						{
+							Generation: two,
+						},
+					},
+					Conditions: []imagev1.TagEventCondition{
+						{
+							Status:     corev1.ConditionFalse,
+							Generation: two,
+						},
+					},
+				},
+				{
+					Tag: "2",
+					Items: []imagev1.TagEvent{
+						{
+							Generation: two,
+						},
+					},
+				},
+			},
+		},
+	}
+	h, cfg, _ := setup()
+	mimic(&h, x86ContentRootDir)
+	dir := h.GetBaseDir(v1.X86Architecture, cfg)
+	files, _ := h.Filefinder.List(dir)
+	h.processFiles(dir, files, cfg)
+	importError := util.Condition(cfg, v1.ImportImageErrorsExist)
+	importError.Status = corev1.ConditionTrue
+	cm := &corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "foo",
+		},
+		Data: map[string]string{
+			"1": "could not import",
+			"2": "could not import",
+		},
+	}
+	fakecmclient := h.configmapclientwrapper.(*fakeConfigMapClientWrapper)
+	fakecmclient.configMaps["foo"] = cm
+	util.ConditionUpdate(cfg, importError)
+	err := h.processImageStreamWatchEvent(stream, false)
+	if err != nil {
+		t.Fatalf("processImageStreamWatchEvent error %#v", err)
+	}
+	cm, stillHasCM := fakecmclient.configMaps["foo"]
+	if !stillHasCM {
+		t.Fatalf("partially clean imagestream did not keep configmap")
+	}
+	_, stillHasCleanTag := cm.Data["2"]
+	if stillHasCleanTag {
+		t.Fatalf("partially clean imagestream still has clean tag in configmap")
+	}
+	_, stillHasBrokenTag := cm.Data["1"]
+	if !stillHasBrokenTag {
+		t.Fatalf("partially clean imagestream does not have tag that is still broke in configmap")
 	}
 }
 
@@ -1086,23 +1183,31 @@ func TestImageStreamImportErrorRecovery(t *testing.T) {
 	h.processFiles(dir, files, cfg)
 	importError := util.Condition(cfg, v1.ImportImageErrorsExist)
 	importError.Status = corev1.ConditionTrue
-	importError.Reason = "foo "
-	importError.Message = "<imagestream/foo> import failed <imagestream/foo>"
+	cm := &corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "foo",
+		},
+		Data: map[string]string{"foo": "could not import"},
+	}
+	fakecmclient := h.configmapclientwrapper.(*fakeConfigMapClientWrapper)
+	fakecmclient.configMaps["foo"] = cm
 	util.ConditionUpdate(cfg, importError)
 	err := h.processImageStreamWatchEvent(stream, false)
 	if err != nil {
 		t.Fatalf("processImageStreamWatchEvent error %#v", err)
 	}
+	_, stillHasCM := fakecmclient.configMaps["foo"]
+	if stillHasCM {
+		t.Fatalf("clean imagestream did not delete configmap")
+	}
+	event := util.Event{
+		Deleted: true,
+		Object:  cm,
+	}
+	h.Handle(event)
 	if util.ConditionTrue(cfg, v1.ImportImageErrorsExist) {
 		t.Fatalf("processImageStreamWatchEvent did not set import error to false %#v", cfg)
 	}
-	importErr := util.Condition(cfg, v1.ImportImageErrorsExist)
-	if len(importErr.Reason) > 0 && util.NameInReason(cfg, importErr.Reason, stream.Name) {
-		t.Fatalf("processImageStreamWatchEvent did not set import error reason field %#v", cfg)
-	}
-	if len(importErr.Message) > 0 {
-		t.Fatalf("processImageStreamWatchEvent did not set import error message field %#v", cfg)
-	}
 }
 
 func TestImageImportRequestCreation(t *testing.T) {
@@ -1158,7 +1263,7 @@ func TestTemplateGetError(t *testing.T) {
 		faketclient := h.templateclientwrapper.(*fakeTemplateClientWrapper)
 		faketclient.geterrors = map[string]error{"bo": terr}
 
-		statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+		statuses := []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 		err := h.Handle(event)
 		if !kerrors.IsNotFound(terr) {
 			statuses[0] = corev1.ConditionUnknown
@@ -1199,14 +1304,11 @@ func TestSameCR(t *testing.T) {
 	mimic(&h, x86ContentRootDir)
 	cfg.ResourceVersion = "a"
 
-	// first pass on the resource creates the samples, exists (first entry, index 0) is still false
-	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+	statuses := []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	err := h.Handle(event)
 	validate(true, err, "", cfg, conditions, statuses, t)
 
 	err = h.Handle(event)
-	// next pass is when we expect exists to be true
-	statuses[0] = corev1.ConditionTrue
 	validate(true, err, "", cfg, conditions, statuses, t)
 
 	err = h.Handle(event)
@@ -1246,7 +1348,7 @@ func TestBadTopLevelStatus(t *testing.T) {
 	err := h.Handle(event)
 	// with deferring sdk updates to the very end, the local object will still have valid statuses on it, even though the error
 	// error returned by h.Handle indicates etcd was not updated
-	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
+	statuses := []corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(false, err, "badsdkupdate", cfg, conditions, statuses, t)
 }
 
@@ -1268,7 +1370,7 @@ func getTKeys() []string {
 }
 
 func mimic(h *Handler, topdir string) {
-	cache.ClearUpsertsCache()
+	//cache.ClearUpsertsCache()
 	registry1 := "registry.access.redhat.com"
 	registry2 := "registry.redhat.io"
 	fakefile := h.Filefinder.(*fakeResourceFileLister)
@@ -1456,6 +1558,10 @@ func NewTestHandler() Handler {
 	h.cvowrapper = operator.NewClusterOperatorHandler(nil)
 	h.cvowrapper.ClusterOperatorWrapper = &cvowrapper
 
+	h.configmapclientwrapper = &fakeConfigMapClientWrapper{
+		configMaps: map[string]*corev1.ConfigMap{},
+	}
+
 	h.skippedImagestreams = make(map[string]bool)
 	h.skippedTemplates = make(map[string]bool)
 
@@ -1480,6 +1586,7 @@ func NewTestHandler() Handler {
 
 	h.imagestreamFile = make(map[string]string)
 	h.templateFile = make(map[string]string)
+	h.imagestreatagToImage = make(map[string]string)
 	h.imagestreamRetry = make(map[string]metav1.Time)
 	h.version = TestVersion
 
@@ -1641,6 +1748,40 @@ func (f *fakeImageStreamImporter) Create(ctx context.Context, isi *imagev1.Image
 	return &imagev1.ImageStreamImport{}, nil
 }
 
+type fakeConfigMapClientWrapper struct {
+	configMaps map[string]*corev1.ConfigMap
+}
+
+func (g *fakeConfigMapClientWrapper) Get(name string) (*corev1.ConfigMap, error) {
+	cm, ok := g.configMaps[name]
+	if !ok {
+		return nil, kerrors.NewNotFound(corev1.Resource("configmap"), name)
+	}
+	return cm, nil
+}
+
+func (g *fakeConfigMapClientWrapper) List() ([]*corev1.ConfigMap, error) {
+	cmList := []*corev1.ConfigMap{}
+	for _, cm := range g.configMaps {
+		cmList = append(cmList, cm)
+	}
+	return cmList, nil
+}
+
+func (g *fakeConfigMapClientWrapper) Create(cm *corev1.ConfigMap) (*corev1.ConfigMap, error) {
+	g.configMaps[cm.Name] = cm
+	return cm, nil
+}
+
+func (g *fakeConfigMapClientWrapper) Update(cm *corev1.ConfigMap) (*corev1.ConfigMap, error) {
+	return g.Create(cm)
+}
+
+func (g *fakeConfigMapClientWrapper) Delete(name string) error {
+	delete(g.configMaps, name)
+	return nil
+}
+
 type fakeTemplateClientWrapper struct {
 	templates    map[string]*templatev1.Template
 	upsertkeys   map[string]bool
